"""
mail.cx 渠道实现
API 文档: https://api.mail.cx/

无需注册，任意邮箱名即可接收邮件
"""

import random
import string
from ..types import EmailInfo, Email
from ..normalize import normalize_email
from .. import http as tm_http

CHANNEL = "mail-cx"
BASE_URL = "https://api.mail.cx/api/v1"
DOMAINS = ["qabq.com", "nqmo.com", "end.tw", "uuf.me", "6n9.net"]

DEFAULT_HEADERS = {
    "Content-Type": "application/json",
    "Accept": "application/json",
    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36",
}


def _random_username(length=8):
    return "".join(random.choices(string.ascii_lowercase + string.digits, k=length))


def _get_token():
    """获取 JWT Token"""
    resp = tm_http.post(
        f"{BASE_URL}/auth/authorize_token",
        headers=DEFAULT_HEADERS,
        json={},
    )
    resp.raise_for_status()
    token = resp.text.strip().strip('"')
    return token


def generate_email():
    """创建临时邮箱"""
    token = _get_token()
    domain = random.choice(DOMAINS)
    username = _random_username()
    email = f"{username}@{domain}"

    return EmailInfo(
        channel=CHANNEL,
        email=email,
        _token=token,
    )


def _extract_email(s):
    """
    从 "name <email>" 格式中提取邮箱地址
    如 "openel <openel@foxmail.com>" → "openel@foxmail.com"
    """
    import re
    if not s:
        return ""
    match = re.search(r"<([^>]+)>", s)
    return match.group(1) if match else s


def _flatten_message(msg, recipient_email):
    """
    将 mail.cx 响应扁平化为 normalize_email 可处理的格式
    mail.cx 的 from 是 "name <email>" 格式，to 是字符串数组
    """
    to_list = msg.get("to", [])
    to_addr = _extract_email(to_list[0]) if isinstance(to_list, list) and to_list else recipient_email

    return {
        "id": msg.get("id", ""),
        "from": _extract_email(msg.get("from", "")),
        "to": to_addr,
        "subject": msg.get("subject", ""),
        "text": msg.get("text", "") or msg.get("body", ""),
        "html": msg.get("html", ""),
        "date": msg.get("date", ""),
        "seen": msg.get("seen", False),
        "attachments": msg.get("attachments", []),
    }


def get_emails(token, email="", **kwargs):
    """获取邮件列表（每次刷新token，mail.cx token有效期仅~5分钟）"""
    fresh_token = _get_token()
    resp = tm_http.get(
        f"{BASE_URL}/mailbox/{email}",
        headers={**DEFAULT_HEADERS, "Authorization": f"Bearer {fresh_token}"},
    )
    resp.raise_for_status()
    messages = resp.json()

    if not isinstance(messages, list) or not messages:
        return []

    emails = []
    for msg in messages:
        try:
            detail_resp = tm_http.get(
                f"{BASE_URL}/mailbox/{email}/{msg['id']}",
                headers={**DEFAULT_HEADERS, "Authorization": f"Bearer {fresh_token}"},
            )
            detail_resp.raise_for_status()
            detail = detail_resp.json()
            emails.append(normalize_email(_flatten_message(detail, email), email))
        except Exception:
            emails.append(normalize_email(_flatten_message(msg, email), email))

    return emails



"""
tempmail.la 渠道实现
API: https://tempmail.la/api
支持分页获取邮件
"""

from .. import http as tm_http
from ..types import EmailInfo, Email
from ..normalize import normalize_email

CHANNEL = "tempmail-la"
BASE_URL = "https://tempmail.la/api"

DEFAULT_HEADERS = {
    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/144.0.0.0 Safari/537.36 Edg/144.0.0.0",
    "Accept": "application/json, text/plain, */*",
    "Content-Type": "application/json",
    "accept-language": "zh-CN,zh;q=0.9,en;q=0.8,en-GB;q=0.7,en-US;q=0.6",
    "cache-control": "no-cache",
    "dnt": "1",
    "locale": "zh-CN",
    "origin": "https://tempmail.la",
    "platform": "PC",
    "pragma": "no-cache",
    "product": "TEMP_MAIL",
    "referer": "https://tempmail.la/zh-CN/tempmail",
    "sec-ch-ua": '"Not(A:Brand";v="8", "Chromium";v="144", "Microsoft Edge";v="144"',
    "sec-ch-ua-mobile": "?0",
    "sec-ch-ua-platform": '"Windows"',
    "sec-fetch-dest": "empty",
    "sec-fetch-mode": "cors",
    "sec-fetch-site": "same-origin",
}


def generate_email(**kwargs) -> EmailInfo:
    """创建临时邮箱"""
    resp = tm_http.post(
        f"{BASE_URL}/mail/create",
        headers=DEFAULT_HEADERS,
        json={"turnstile": ""},
    )
    resp.raise_for_status()
    data = resp.json()

    if data.get("code") != 0 or not data.get("data"):
        raise Exception("Failed to generate email")

    return EmailInfo(
        channel=CHANNEL,
        email=data["data"]["address"],
        expires_at=data["data"].get("endAt"),
        created_at=data["data"].get("startAt"),
    )


def get_emails(email: str, **kwargs) -> list:
    """获取邮件列表（支持分页）"""
    all_emails = []
    cursor = None
    has_more = True

    while has_more:
        resp = tm_http.post(
            f"{BASE_URL}/mail/box",
            headers=DEFAULT_HEADERS,
            json={"address": email, "cursor": cursor},
        )
        resp.raise_for_status()
        data = resp.json()

        if data.get("code") != 0 or not data.get("data"):
            raise Exception("Failed to get emails")

        rows = data["data"].get("rows") or []
        all_emails.extend(rows)

        if data["data"].get("hasMore") and data["data"].get("cursor"):
            cursor = data["data"]["cursor"]
        else:
            has_more = False

    return [normalize_email(raw, email) for raw in all_emails]



"""
dropmail.me 渠道实现
API: GraphQL https://dropmail.me/api/graphql/MY_TOKEN
"""

from .. import http as tm_http
from ..types import EmailInfo, Email
from ..normalize import normalize_email

CHANNEL = "dropmail"
BASE_URL = "https://dropmail.me/api/graphql/MY_TOKEN"

CREATE_SESSION_QUERY = "mutation {introduceSession {id, expiresAt, addresses{id, address}}}"

GET_MAILS_QUERY = """query ($id: ID!) {
  session(id:$id) {
    mails {
      id, rawSize, fromAddr, toAddr, receivedAt,
      text, headerFrom, headerSubject, html
    }
  }
}"""


def _graphql_request(query: str, variables: dict = None) -> dict:
    """执行 GraphQL 请求（application/x-www-form-urlencoded）"""
    data = {"query": query}
    if variables:
        import json
        data["variables"] = json.dumps(variables)

    resp = tm_http.post(BASE_URL, data=data)
    resp.raise_for_status()
    result = resp.json()

    if result.get("errors"):
        raise Exception(f"GraphQL error: {result['errors'][0].get('message', 'Unknown')}")

    return result.get("data", {})


def _flatten_message(mail: dict, recipient_email: str) -> dict:
    """将 dropmail 消息格式扁平化"""
    return {
        "id": mail.get("id", ""),
        "from": mail.get("fromAddr", ""),
        "to": mail.get("toAddr", recipient_email),
        "subject": mail.get("headerSubject", ""),
        "text": mail.get("text", ""),
        "html": mail.get("html", ""),
        "received_at": mail.get("receivedAt", ""),
        "attachments": [],
    }


def generate_email(**kwargs) -> EmailInfo:
    """创建临时邮箱"""
    data = _graphql_request(CREATE_SESSION_QUERY)
    session = data.get("introduceSession")

    if not session or not session.get("addresses"):
        raise Exception("Failed to generate email")

    return EmailInfo(
        channel=CHANNEL,
        email=session["addresses"][0]["address"],
        _token=session["id"],
        expires_at=session.get("expiresAt"),
    )


def get_emails(token: str, email: str = "", **kwargs) -> list:
    """获取邮件列表"""
    data = _graphql_request(GET_MAILS_QUERY, {"id": token})
    mails = (data.get("session") or {}).get("mails") or []
    return [normalize_email(_flatten_message(m, email), email) for m in mails]



"""
maildrop.cc 渠道实现
API: GraphQL https://api.maildrop.cc/graphql
"""

import random
import string
import email
import email.header
import email.policy
from .. import http as tm_http
from ..types import EmailInfo, Email

CHANNEL = "maildrop"
GRAPHQL_URL = "https://api.maildrop.cc/graphql"
DOMAIN = "maildrop.cc"


def _decode_rfc2047(s: str) -> str:
    """
    解码 RFC 2047 编码的邮件头
    如 =?utf-8?B?b3BlbmVs?= 解码为可读文本
    """
    if not s:
        return ""
    decoded_parts = email.header.decode_header(s)
    result = []
    for part, charset in decoded_parts:
        if isinstance(part, bytes):
            result.append(part.decode(charset or "utf-8", errors="replace"))
        else:
            result.append(part)
    return " ".join(result)


def _extract_text_from_mime(raw: str) -> str:
    """
    从原始 MIME 邮件源码中提取纯文本正文
    maildrop 的 data 字段返回完整 MIME 源码，需要解析出 text/plain 部分
    """
    if not raw:
        return ""
    try:
        msg = email.message_from_string(raw, policy=email.policy.default)
        # 优先提取 text/plain 部分
        body = msg.get_body(preferencelist=("plain",))
        if body:
            content = body.get_content()
            return content.strip() if isinstance(content, str) else content.decode("utf-8", errors="replace").strip()
        # 回退到 text/html 并去除标签
        body = msg.get_body(preferencelist=("html",))
        if body:
            content = body.get_content()
            return content.strip() if isinstance(content, str) else content.decode("utf-8", errors="replace").strip()
    except Exception:
        pass
    # 解析失败则尝试跳过邮件头返回正文
    idx = raw.find("\r\n\r\n")
    return raw[idx + 4:].strip() if idx >= 0 else raw


def _random_username(length: int = 10) -> str:
    return "".join(random.choices(string.ascii_lowercase + string.digits, k=length))


def _graphql_request(
    operation_name: str, query: str, variables: dict = None
) -> dict:
    """
    发送 GraphQL 请求
    使用 operationName + variables 的标准 GraphQL 格式
    """
    resp = tm_http.post(
        GRAPHQL_URL,
        headers={
            "Content-Type": "application/json",
            "Origin": "https://maildrop.cc",
            "Referer": "https://maildrop.cc/",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36",
        },
        json={
            "operationName": operation_name,
            "variables": variables or {},
            "query": query,
        },
    )
    resp.raise_for_status()
    result = resp.json()

    if result.get("errors"):
        raise Exception(f"Maildrop GraphQL error: {result['errors'][0].get('message')}")

    return result.get("data", {})


def generate_email(**kwargs) -> EmailInfo:
    """创建临时邮箱，任意用户名即可接收邮件"""
    username = _random_username()
    email = f"{username}@{DOMAIN}"

    # 验证 API 可用
    _graphql_request(
        "GetInbox",
        "query GetInbox($mailbox: String!) { inbox(mailbox: $mailbox) { id } }",
        {"mailbox": username},
    )

    return EmailInfo(
        channel=CHANNEL,
        email=email,
        _token=username,
    )


def get_emails(token: str, email: str = "", **kwargs) -> list:
    """获取邮件列表"""
    mailbox = token or email.split("@")[0]

    # 查询收件箱
    data = _graphql_request(
        "GetInbox",
        "query GetInbox($mailbox: String!) { inbox(mailbox: $mailbox) { id headerfrom subject date } }",
        {"mailbox": mailbox},
    )
    inbox = data.get("inbox") or []
    if not inbox:
        return []

    # 逐封获取完整内容
    emails = []
    for item in inbox:
        try:
            msg_data = _graphql_request(
                "GetMessage",
                "query GetMessage($mailbox: String!, $id: String!) { message(mailbox: $mailbox, id: $id) { id headerfrom subject date data html } }",
                {"mailbox": mailbox, "id": item["id"]},
            )
            msg = msg_data.get("message")
            if msg:
                emails.append(Email(
                    id=msg.get("id") or item.get("id", ""),
                    from_addr=_decode_rfc2047(msg.get("headerfrom") or item.get("headerfrom", "")),
                    to=email,
                    subject=_decode_rfc2047(msg.get("subject") or item.get("subject", "")),
                    text=_extract_text_from_mime(msg.get("data", "")),
                    html=msg.get("html", ""),
                    date=msg.get("date") or item.get("date", ""),
                    is_read=False,
                    attachments=[],
                ))
        except Exception:
            pass

    return emails
